94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
187 lines
5.2 KiB
Python
187 lines
5.2 KiB
Python
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
from sglang.srt.lora.utils import LoRABatchInfo
|
|
|
|
|
|
@triton.jit
|
|
def _embedding_lora_a_kernel(
|
|
# Pointers to tensors
|
|
input_ids,
|
|
weights,
|
|
output,
|
|
extra_embeddings,
|
|
# Dimensions
|
|
vocab_size,
|
|
rank,
|
|
num_loras,
|
|
# Strides
|
|
w_stride_0, # stride for lora index
|
|
w_stride_1, # stride for rank
|
|
w_stride_2, # stride for vocab
|
|
output_stride_0,
|
|
output_stride_1,
|
|
extra_emb_stride_0, # stride for lora index
|
|
extra_emb_stride_1, # stride for token
|
|
extra_emb_stride_2, # stride for hidden dim (= rank for extra embeddings)
|
|
# Batch info
|
|
seg_lens,
|
|
seg_indptr,
|
|
weight_indices,
|
|
lora_ranks,
|
|
# Meta-parameters
|
|
BLOCK_RANK: tl.constexpr,
|
|
HAS_EXTRA_EMBEDDINGS: tl.constexpr,
|
|
):
|
|
"""
|
|
Embedding lookup for LoRA A weights with support for extra tokens.
|
|
|
|
Each program handles one token across a block of rank dimensions.
|
|
Grid: (cdiv(max_len, 1), bs) - one program per token in each batch
|
|
"""
|
|
batch_id = tl.program_id(axis=1)
|
|
token_idx = tl.program_id(axis=0)
|
|
|
|
w_index = tl.load(weight_indices + batch_id)
|
|
rank_val = tl.load(lora_ranks + w_index)
|
|
|
|
# If rank is 0, skip
|
|
if rank_val == 0:
|
|
return
|
|
|
|
seg_start = tl.load(seg_indptr + batch_id)
|
|
seg_len = tl.load(seg_lens + batch_id)
|
|
|
|
# Check if this token is within the segment
|
|
if token_idx >= seg_len:
|
|
return
|
|
|
|
# Load the token ID
|
|
token_id = tl.load(input_ids + seg_start + token_idx)
|
|
|
|
# Process in chunks of BLOCK_RANK dimensions
|
|
num_blocks = tl.cdiv(rank_val, BLOCK_RANK)
|
|
|
|
for block_id in range(num_blocks):
|
|
rank_offset = tl.arange(0, BLOCK_RANK) + block_id * BLOCK_RANK
|
|
rank_mask = rank_offset < rank_val
|
|
|
|
# Check if this is an extra token
|
|
is_extra_token = token_id >= vocab_size
|
|
|
|
if HAS_EXTRA_EMBEDDINGS and is_extra_token:
|
|
# Use extra embeddings
|
|
extra_token_id = token_id - vocab_size
|
|
extra_emb_ptr = (
|
|
extra_embeddings
|
|
+ w_index * extra_emb_stride_0
|
|
+ extra_token_id * extra_emb_stride_1
|
|
+ rank_offset * extra_emb_stride_2
|
|
)
|
|
emb_values = tl.load(extra_emb_ptr, mask=rank_mask, other=0.0)
|
|
else:
|
|
# Use regular LoRA A weights
|
|
# weights shape: (num_loras, rank, vocab_size)
|
|
# We need to load weights[w_index, rank_offset, token_id]
|
|
token_id_clamped = tl.minimum(token_id, vocab_size - 1)
|
|
weight_ptr = (
|
|
weights
|
|
+ w_index * w_stride_0
|
|
+ rank_offset * w_stride_1
|
|
+ token_id_clamped * w_stride_2
|
|
)
|
|
emb_values = tl.load(weight_ptr, mask=rank_mask, other=0.0)
|
|
|
|
# Write to output
|
|
output_ptr = (
|
|
output
|
|
+ (seg_start + token_idx) * output_stride_0
|
|
+ rank_offset * output_stride_1
|
|
)
|
|
tl.store(output_ptr, emb_values, mask=rank_mask)
|
|
|
|
|
|
def embedding_lora_a_fwd(
|
|
input_ids: torch.Tensor,
|
|
weights: torch.Tensor,
|
|
batch_info: LoRABatchInfo,
|
|
vocab_size: int,
|
|
extra_embeddings: torch.Tensor = None,
|
|
) -> torch.Tensor:
|
|
"""
|
|
Forward pass for LoRA A embedding lookup.
|
|
|
|
Args:
|
|
input_ids: (s,) token IDs
|
|
weights: (num_loras, rank, vocab_size) LoRA A embedding weights
|
|
batch_info: LoRABatchInfo containing batch information
|
|
vocab_size: base vocabulary size
|
|
extra_embeddings: (num_loras, num_extra_tokens, rank) extra token embeddings
|
|
|
|
Returns:
|
|
output: (s, rank) embedded features
|
|
"""
|
|
assert input_ids.is_contiguous()
|
|
assert weights.is_contiguous()
|
|
assert len(input_ids.shape) == 1
|
|
assert len(weights.shape) == 3
|
|
|
|
S = input_ids.shape[0]
|
|
num_loras = weights.shape[0]
|
|
rank = weights.shape[1]
|
|
vocab_size_weights = weights.shape[2]
|
|
|
|
# Block size for rank dimension
|
|
BLOCK_RANK = 128
|
|
|
|
has_extra_embeddings = extra_embeddings is not None
|
|
|
|
if has_extra_embeddings:
|
|
assert extra_embeddings.is_contiguous()
|
|
extra_emb_stride = (
|
|
extra_embeddings.stride(0),
|
|
extra_embeddings.stride(1),
|
|
extra_embeddings.stride(2),
|
|
)
|
|
else:
|
|
# Create dummy tensor to satisfy Triton
|
|
extra_embeddings = torch.empty(
|
|
(1, 1, 1), device=input_ids.device, dtype=weights.dtype
|
|
)
|
|
extra_emb_stride = (1, 1, 1)
|
|
|
|
# Grid: one program per token in each batch segment
|
|
grid = (
|
|
batch_info.max_len,
|
|
batch_info.bs,
|
|
)
|
|
|
|
output = torch.zeros((S, rank), device=input_ids.device, dtype=weights.dtype)
|
|
|
|
_embedding_lora_a_kernel[grid](
|
|
input_ids,
|
|
weights,
|
|
output,
|
|
extra_embeddings,
|
|
vocab_size,
|
|
rank,
|
|
num_loras,
|
|
weights.stride(0),
|
|
weights.stride(1),
|
|
weights.stride(2),
|
|
output.stride(0),
|
|
output.stride(1),
|
|
extra_emb_stride[0],
|
|
extra_emb_stride[1],
|
|
extra_emb_stride[2],
|
|
batch_info.seg_lens,
|
|
batch_info.seg_indptr,
|
|
batch_info.weight_indices,
|
|
batch_info.lora_ranks,
|
|
BLOCK_RANK,
|
|
has_extra_embeddings,
|
|
)
|
|
|
|
return output
|